<?php

namespace PixelRockstar\Settings;

use PixelRockstar\PixelRockstar;

class Settings
{
    /** @var array */
    private static $settings;

    /**
     * @param string $name
     * @param mixed  $value
     *
     * @return bool
     */
    public static function setSetting($name, $value)
    {
        $settings = array($name => $value);

        return self::setSettings($settings);
    }

    /**
     * @param string $name
     * @param string $default
     * @param bool   $notEmpty If `true` and value is empty, `$default` is returned
     *
     * @return mixed
     */
    public static function getSetting($name, $default = "", $notEmpty = false)
    {
        $settings = self::getSettings();

        if (!isset($settings[$name]) || ($notEmpty && $settings[$name] === "")) {
            return $default;
        }

        return $settings[$name];
    }

    /**
     * @param array $settings
     * @param bool  $overwrite If `true`, current data are overwritten with `$data`,
     *                         otherwise old and new values are merged
     *
     * @return bool
     */
    public static function setSettings($settings, $overwrite = false)
    {
        if (!is_array($settings)) {
            return false;
        }

        if (!$overwrite) {
            $settings = array_merge(self::getSettings(), $settings);
        }

        $sanitizedSettings = array();
        foreach ($settings as $key => $value) {
            $sanitizedSettings[sanitize_text_field($key)] = sanitize_text_field($value);
        }

        self::$settings = $sanitizedSettings;

        return update_option(self::getSettingsKey(), $sanitizedSettings);
    }

    /**
     * @return array
     */
    public static function getSettings()
    {
        if (!is_array(self::$settings)) {
            self::$settings = get_option(self::getSettingsKey(), array());
        }

        return self::$settings;
    }

    /**
     * @param string $key
     *
     * @return bool
     */
    public static function removeSetting($key)
    {
        return self::removeSettings(array($key));
    }

    /**
     * @param array $keys
     *
     * @return bool
     */
    public static function removeSettings(array $keys)
    {
        $settings = self::getSettings();
        foreach ($keys as $key) {
            unset($settings[$key]);
        }

        return self::setSettings($settings, true);
    }

    /**
     * @return string
     */
    private static function getSettingsKey()
    {
        return PixelRockstar::$appId . "_settings";
    }
}
