<?php
namespace PixelRockstar\User;

class Subscription
{
    const RENEWAL_DAILY = "daily";
    const RENEWAL_MONTHLY = "monthly";
    const RENEWAL_YEARLY = "yearly";

    /** @var int Downloads available in user's subscription package this billing period */
    private $downloadsInPackage = 0;

    /** @var int Downloads used this billing period */
    private $downloadsUsed = 0;

    /** @var int Downloads available in this billing period */
    private $downloadsAvailable = 0;

    /** @var int Credits available */
    private $creditsAvailable = 0;

    /** @var \DateTime Date of the first day of subscription package */
    private $startDate = 0;

    /** @var \DateTime Date of the last day of subscription package */
    private $endDate = 0;

    /** @var string */
    private $renewalPeriod;

    /** @var string */
    private $packageName;

    /**
     * @return int
     */
    public function getDownloadsInPackage()
    {
        return $this->downloadsInPackage;
    }

    /**
     * @param int $downloadsInPackage
     *
     * @return $this
     */
    public function setDownloadsInPackage($downloadsInPackage)
    {
        $this->downloadsInPackage = $downloadsInPackage;

        return $this;
    }

    /**
     * @return int
     */
    public function getDownloadsUsed()
    {
        return $this->downloadsUsed;
    }

    /**
     * @param int $downloadsUsed
     *
     * @return $this
     */
    public function setDownloadsUsed($downloadsUsed)
    {
        $this->downloadsUsed = $downloadsUsed;

        return $this;
    }

    /**
     * @return int
     */
    public function getDownloadsAvailable()
    {
        return $this->downloadsAvailable;
    }

    /**
     * @param int $downloadsAvailable
     *
     * @return $this
     */
    public function setDownloadsAvailable($downloadsAvailable)
    {
        $this->downloadsAvailable = $downloadsAvailable;

        return $this;
    }

    /**
     * @return int
     */
    public function getCreditsAvailable()
    {
        return $this->creditsAvailable;
    }

    /**
     * @param int $creditsAvailable
     *
     * @return $this
     */
    public function setCreditsAvailable($creditsAvailable)
    {
        $this->creditsAvailable = $creditsAvailable;

        return $this;
    }

    /**
     * @return \DateTime
     */
    public function getStartDate()
    {
        return $this->startDate;
    }

    /**
     * @param string $format Format of the returned value in `date()` format
     *
     * @return mixed
     */
    public function getStartDateFormatted($format = "U")
    {
        return $this->startDate->format($format);
    }

    /**
     * @param \DateTime $startDate
     *
     * @return $this
     */
    public function setStartDate(\DateTime $startDate)
    {
        $this->startDate = $startDate;

        return $this;
    }

    /**
     * @return \DateTime
     */
    public function getEndDate()
    {
        return $this->endDate;
    }

    /**
     * @param string $format Format of the returned value in `date()` format
     *
     * @return mixed
     */
    public function getEndDateFormatted($format = "U")
    {
        if (!($this->endDate instanceof \DateTime)) {
            return null;
        }

        return $this->endDate->format($format);
    }

    /**
     * @return int
     */
    public function getDaysUntilEnd()
    {
        $now = new \DateTime();
        if (!($this->getEndDate() instanceof \DateTime)) {
            return 0;
        }

        return $now->diff($this->getEndDate(), true)->days;
    }

    /**
     * @param \DateTime|int $endDate
     *
     * @return $this
     */
    public function setEndDate($endDate)
    {
        if (!($endDate instanceof \DateTime)) {
            $endDate = \DateTime::createFromFormat("U", $endDate);
        }

        $this->endDate = $endDate;
        return $this;
    }

    /**
     * @return string
     */
    public function getRenewalPeriod()
    {
        return $this->renewalPeriod;
    }

    /**
     * @param string $renewalPeriod
     *
     * @return $this
     */
    public function setRenewalPeriod($renewalPeriod)
    {
        $this->renewalPeriod = $renewalPeriod;

        return $this;
    }

    /**
     * @return string
     */
    public function getPackageName()
    {
        return $this->packageName;
    }

    /**
     * @param string $packageName
     *
     * @return $this
     */
    public function setPackageName($packageName)
    {
        $this->packageName = $packageName;

        return $this;
    }
}
