const PIXELROCKSTAR_AJAX_ADMIN_ACTION = "pixelrockstar_admin_action";
const PIXELROCKSTAR_BUTTON_SAVE = ".js-pixelrockstar-button-save";
const PIXELROCKSTAR_UNSAVED_CHANGES_POPUP = ".js-pixelrockstar-unsaved-changes-dialog";
const PIXELROCKSTAR_UNSAVED_CHANGES_LEAVE = ".js-pixelrockstar-unsaved-changes-leave";
const PIXELROCKSTAR_UNSAVED_CHANGES_STAY = ".js-pixelrockstar-unsaved-changes-stay";

// Login box
const PIXELROCKSTAR_DIV_LOGGED_OUT = ".js-pixelrockstar-logged-out";
const PIXELROCKSTAR_DIV_LOGGED_IN = ".js-pixelrockstar-logged-in";
const PIXELROCKSTAR_FORM_NO_SUBMIT = ".js-pixelrockstar-no-submit";
const PIXELROCKSTAR_FORM_SUBMIT = ".js-pixelrockstar-submit";
const PIXELROCKSTAR_SPAN_LOGGED_AS = ".js-pixelrockstar-username";
const PIXELROCKSTAR_INPUT_APIKEY = "#pixelrockstar_apikey";
const PIXELROCKSTAR_BUTTON_LOGIN = ".js-pixelrockstar-login-button";
const PIXELROCKSTAR_BUTTON_LOGOUT = ".js-pixelrockstar-logout-button";
const PIXELROCKSTAR_LOGIN_STATUS = ".js-pixelrockstar-login-status";

// Logged in box
const PIXELROCKSTAR_DOWNLOADS_REMAINING = ".js-pixelrockstar-downloads-remaining";
const PIXELROCKSTAR_DAYS_REMAINING = ".js-pixelrockstar-days-remaining";
const PIXELROCKSTAR_DAYS_REMAINING_CONTAINER = ".js-pixelrockstar-days-remaining-container";
const PIXELROCKSTAR_EXPIRED_SUBSCRIPTION = ".js-pixelrockstar-expired-subscription";
const PIXELROCKSTAR_BUY_CREDITS_CONTAINER = ".js-pixelrockstar-buy-downloads-container";

// Settings box
const PIXELROCKSTAR_SAVE_SETTINGS_BUTTON = ".js-pixelrockstar-save-settings-button";
const PIXELROCKSTAR_SETTINGS_FORM = ".js-pixelrockstar-save-settings";
const PIXELROCKSTAR_SETTINGS_STATUS = ".js-pixelrockstar-settings-status";
const PIXELROCKSTAR_INPUT_CUSTOM_SIZE = ".js-custom-size-input";
const PIXELROCKSTAR_RADIO_CUSTOM_SIZE = ".js-custom-size-radio";
const PIXELROCKSTAR_INPUT_MEDIA_SIZE = ".js-media-size-input";
const PIXELROCKSTAR_RADIO_MEDIA_SIZE = ".js-media-size-radio";

// Advanced settings
const PIXELROCKSTAR_INPUT_CUSTOM_CAPTION = ".js-custom-caption-input";
const PIXELROCKSTAR_RADIO_CUSTOM_CAPTION = ".js-custom-caption-radio";

var goToUrl = "";
var formDefaults = "";

jQuery(document).ready(function ($) {
  formDefaults = pixelrockstar_formsValuesSerialized();

  // Set form changed bit
  $(".pixelrockstar form").on("change", function () {
    var $this = $(this);
    if ($this.attr("id") === "pixelrockstar_login_form") {
      return;
    }

    if (pixelrockstar_formChanged()) {
      $(PIXELROCKSTAR_BUTTON_SAVE).removeClass("pixelrockstar-u-isDisabled");
    } else {
      $(PIXELROCKSTAR_BUTTON_SAVE).addClass("pixelrockstar-u-isDisabled");
    }
  });

  // Display notification if leaving page without saving changes
  $("a").on("click", function (e) {
    var href = $(this).attr("href");
    if (!pixelrockstar_formChanged() || goToUrl || !href) {
      return;
    }

    try {
      $(PIXELROCKSTAR_UNSAVED_CHANGES_POPUP).removeClass("pixelrockstar-u-isHidden");
      goToUrl = href;
      e.preventDefault();
    } catch (ex) {
      // no popup to be displayed, then don't care
    }
  });

  $(PIXELROCKSTAR_UNSAVED_CHANGES_LEAVE).on("click", function () {
    $(PIXELROCKSTAR_UNSAVED_CHANGES_POPUP).addClass("pixelrockstar-u-isHidden");
    if (goToUrl) {
      pixelrockstar_showLoader("large", "black");
      window.location.href = goToUrl;
    }
  });

  $(PIXELROCKSTAR_UNSAVED_CHANGES_STAY).on("click", function () {
    $(PIXELROCKSTAR_UNSAVED_CHANGES_POPUP).addClass("pixelrockstar-u-isHidden");
    goToUrl = "";
  });

  // Don't submit AJAX forms
  $(PIXELROCKSTAR_FORM_NO_SUBMIT).on("submit", function (e) {
    e.preventDefault();
  });

  $(PIXELROCKSTAR_FORM_SUBMIT).on("submit", function (e) {
    var $this = $(this);
    var $button = $this.find(PIXELROCKSTAR_BUTTON_SAVE).first();

    e.preventDefault();
    try {
      $button.trigger("click");
    } catch (ex) {
    }
  });

  // Validate API key and log in if possible
  $(PIXELROCKSTAR_BUTTON_LOGIN).on("click", function () {
    var $apikey = $(PIXELROCKSTAR_INPUT_APIKEY);
    var apikey = $apikey.val();

    if (apikey.length !== 40) {
      $apikey.addClass("has-rror");
      pixelrockstar_displayError(PIXELROCKSTAR_LOGIN_STATUS, "Invalid API key.");
      return;
    }

    pixelrockstar_hideMessage(PIXELROCKSTAR_LOGIN_STATUS);
    pixelrockstar_showLoader("large", "black");

    var data = {
      "action": PIXELROCKSTAR_AJAX_ADMIN_ACTION,
      "what": "login",
      "apikey": apikey
    };

    $.post(ajaxurl, data, function (response) {
      pixelrockstar_hideLoader();

      if (response.success == true) {
        $(PIXELROCKSTAR_SPAN_LOGGED_AS).html(response.username);
        $(PIXELROCKSTAR_DOWNLOADS_REMAINING).html(response.downloads_remaining);
        $(PIXELROCKSTAR_DAYS_REMAINING).html(response.days_remaining);
        $(PIXELROCKSTAR_DIV_LOGGED_OUT).addClass("pixelrockstar-u-isHidden");
        $(PIXELROCKSTAR_DIV_LOGGED_IN).removeClass("pixelrockstar-u-isHidden");

        if (response.days_remaining > 0) {
          $(PIXELROCKSTAR_DAYS_REMAINING_CONTAINER).removeClass("pixelrockstar-u-isHidden");
          $(PIXELROCKSTAR_BUY_CREDITS_CONTAINER).removeClass("pixelrockstar-u-isHidden");
          $(PIXELROCKSTAR_EXPIRED_SUBSCRIPTION).addClass("pixelrockstar-u-isHidden");
        } else {
          $(PIXELROCKSTAR_DAYS_REMAINING_CONTAINER).addClass("pixelrockstar-u-isHidden");
          $(PIXELROCKSTAR_BUY_CREDITS_CONTAINER).addClass("pixelrockstar-u-isHidden");
          $(PIXELROCKSTAR_EXPIRED_SUBSCRIPTION).removeClass("pixelrockstar-u-isHidden");
        }

        $apikey.val("");
      } else {
        pixelrockstar_displayError(PIXELROCKSTAR_LOGIN_STATUS, response.error);
      }
    }, "json");
  });

  // Log out
  $(PIXELROCKSTAR_BUTTON_LOGOUT).on("click", function (e) {
    var data = {
      "action": PIXELROCKSTAR_AJAX_ADMIN_ACTION,
      "what": "logout"
    };

    e.preventDefault();

    pixelrockstar_hideMessage(PIXELROCKSTAR_LOGIN_STATUS);
    pixelrockstar_showLoader("large", "black");

    $.post(ajaxurl, data, function (response) {
      pixelrockstar_hideLoader();

      if (response.success == true) {
        $(PIXELROCKSTAR_DIV_LOGGED_IN).addClass("pixelrockstar-u-isHidden");
        $(PIXELROCKSTAR_DIV_LOGGED_OUT).removeClass("pixelrockstar-u-isHidden");
      } else {
        pixelrockstar_displayError(PIXELROCKSTAR_LOGIN_STATUS, response.error);
      }
    }, "json");
  });

  // Limit input value for custom image size
  $(PIXELROCKSTAR_INPUT_CUSTOM_SIZE).on("input keydown keyup", function () {
    var $this = $(this);
    var value;
    var max;

    value = $this.val().replace(/\D/g, "");
    max = parseInt($this.attr("max"));
    if (isNaN(parseInt(value)) || isNaN(max)) {
      $this.val(value);
      return;
    }

    if (parseInt(value) > max) {
      value = max;
    }

    $this.val(value);
    $(PIXELROCKSTAR_RADIO_CUSTOM_SIZE).attr("checked", "checked");
  });

  $(PIXELROCKSTAR_INPUT_MEDIA_SIZE).on("change", function () {
    $(PIXELROCKSTAR_RADIO_MEDIA_SIZE).attr("checked", "checked");
  });

  $(PIXELROCKSTAR_SETTINGS_FORM).on("change", function () {
    pixelrockstar_hideMessage(PIXELROCKSTAR_SETTINGS_STATUS);
  });

  $(PIXELROCKSTAR_INPUT_CUSTOM_CAPTION).on("input keydown keyup", function () {
    $(PIXELROCKSTAR_RADIO_CUSTOM_CAPTION).attr("checked", "checked");
  });

  // Save settings
  $(PIXELROCKSTAR_SAVE_SETTINGS_BUTTON).on("click", function (e) {
    var $this = $(this);
    var $form = $(PIXELROCKSTAR_SETTINGS_FORM);
    var data = {
      "action": PIXELROCKSTAR_AJAX_ADMIN_ACTION,
      "what": "update-settings",
      "data": $form.serialize()
    };

    e.preventDefault();

    if ($this.hasClass("pixelrockstar-u-isDisabled")) {
      return;
    }

    pixelrockstar_hideMessage(PIXELROCKSTAR_LOGIN_STATUS);
    pixelrockstar_showLoader("large", "black");

    $.post(ajaxurl, data, function (response) {
      pixelrockstar_hideLoader();

      if (response.success == true) {
        pixelrockstar_displayMessage(PIXELROCKSTAR_SETTINGS_STATUS, response.message);

        formDefaults = pixelrockstar_formsValuesSerialized();
        $this.addClass("pixelrockstar-u-isDisabled");
      } else {
        pixelrockstar_displayError(PIXELROCKSTAR_SETTINGS_STATUS, response.error);
      }
    }, "json");

  });
});

function pixelrockstar_displayMessage(target, message) {
  pixelrockstar_hideMessage(target);
  jQuery(target)
    .removeClass("pixelrockstar-u-isHidden")
    .addClass("pixelrockstar-u-isOk")
    .html(message);
}

function pixelrockstar_displayError(target, message) {
  pixelrockstar_hideMessage(target);
  jQuery(target)
    .removeClass("pixelrockstar-u-isHidden")
    .addClass("pixelrockstar-u-isError")
    .html(message);
}

function pixelrockstar_hideMessage(target) {
  jQuery(target)
    .removeClass("pixelrockstar-u-isError pixelrockstar-u-isOk")
    .addClass("pixelrockstar-u-isHidden");
}

function pixelrockstar_formsValuesSerialized() {
  var $forms = jQuery(".pixelrockstar form");
  var serialized = "";

  $forms.each(function () {
    var $form = jQuery(this);
    serialized += $form.serialize()
  });

  return serialized;
}

function pixelrockstar_formChanged() {
  return (pixelrockstar_formsValuesSerialized() !== formDefaults);
}