const PIXELROCKSTAR_AJAX_ACTION = "pixelrockstar_action";

const PIXELROCKSTAR_LOADER_ID = "pixelrockstar-loader";
const PIXELROCKSTAR_CLOSE_DIALOG = ".js-pixelrockstar-closeDialog";
const PIXELROCKSTAR_DIV_SEARCH_RESULTS = ".pixelrockstar-searchResults";

// Edit page
const PIXELROCKSTAR_BUTTON_ADD_STOCK_PHOTO = ".js-pixelrockstar-add-stock-photo";
const PIXELROCKSTAR_BUTTON_INSERT_MEDIA = ".insert-media";

// Value in <select> for custom sizes
const PIXELROCKSTAR_CUSTOM_SIZE_VALUE = "custom";

jQuery(document).ready(function ($) {
  // "Pixelrockstar" button on Edit Post page
  $(PIXELROCKSTAR_BUTTON_ADD_STOCK_PHOTO).on('click', function (event) {
    var $this = $(this);
    $this.attr("data-button-clicked", "1");

    event.preventDefault();

    return $(PIXELROCKSTAR_BUTTON_INSERT_MEDIA).trigger('click');
  });

  $("body").on('click', PIXELROCKSTAR_CLOSE_DIALOG, function (event) {
    event.preventDefault();

    pixelrockstar_closeDialog();
  });

  pixelrockstar_uploadMediaHook();
});

/**
 * Adds an overlay loader
 *
 * @param size  (string) Must match modifier name of `loader--` (e.g. "small" / "large")
 * @param color (string) Must match modifier name of `loader--` (e.g. "black" / "white")
 * @param $element (object) jQuery object in which should be loader displayed; if not set, loader will be over whole page
 */
function pixelrockstar_showLoader(size, color, $element) {
  var $ = jQuery;
  var fullscreen = false;

  if (typeof $element === "undefined") {
    $element = $(".pixelrockstar").first();
    if (typeof $element === "undefined") {
      $element = $("body");
    }

    fullscreen = true;
  }

  // Create an overlay element
  var $overlay = $("<div>")
    .addClass("pixelrockstar-Overlay")
    .attr("id", PIXELROCKSTAR_LOADER_ID);

  if (fullscreen) {
    $overlay.addClass("pixelrockstar-Overlay--Fullscreen");
  }

  // Create a spinner element
  var $loader = $("<em>")
    .addClass("pixelrockstar-Loader pixelrockstar-Loader--" + size + " pixelrockstar-Loader--" + color);

  pixelrockstar_hideLoader();

  $loader.appendTo($overlay);
  $overlay.appendTo($element);
}

/**
 * Removes loader
 */
function pixelrockstar_hideLoader() {
  try {
    jQuery("#" + PIXELROCKSTAR_LOADER_ID).remove();
  } catch (e) {
  }
}

/**
 * Displays a dialog
 *
 * @param texts An object with properties from CSS class pixelrockstar-Dialog-Content--*
 *                 E.g. { top: "Text in top section", bottom: "Text in bottom section" }
 * @param settings Dialog modifiers. Supported values:
 *                 overlay {
 *                   isDark: [bool],        // whether to use light or dark theme of overlay
 *                   isFullscreen: [bool]   // whether to display the overlay over whole page
 *                 }
 */
function pixelrockstar_dialog(texts, settings) {
  var $ = jQuery;

  var $container = $("<div/>", {
    'class': "pixelrockstar-Dialog-Container"
  });
  var $content = $("<div/>", {
    'class': "pixelrockstar-Dialog-Content pixelrockstar-MinWidth--500"
  });

  settings = settings || {
    overlay: {
      isDark: true,
      isFullscreen: true
    }
  };

  if (settings.class) {
    $container.addClass(settings.class);
  }

  if (settings.overlay) {
    $container.addClass("pixelrockstar-Overlay");
    if (settings.overlay.isDark) {
      $container.addClass("pixelrockstar-Overlay--Dark");
    }
    if (settings.overlay.isFullscreen) {
      $container.addClass("pixelrockstar-Overlay--Fullscreen");
    }
  }

  for (var key in texts) {
    if (!texts.hasOwnProperty(key)) {
      continue;
    }

    $("<div/>", {
      'class': "pixelrockstar-Dialog-Content--" + key,
      'html': texts[key]
    }).appendTo($content);
  }

  pixelrockstar_closeDialog();

  $content.appendTo($container);
  $container.appendTo("body");
}

/**
 * Closes dialog
 */
function pixelrockstar_closeDialog() {
  jQuery(".pixelrockstar-Dialog-Container").remove();
}

/********************
 * Gallery plugin ***
 ********************/

function pixelrockstar_renderGallery() {
  var $container = jQuery(PIXELROCKSTAR_DIV_SEARCH_RESULTS);
  if ($container.length === 0) {
    return;
  }

  try {
    $container
      .justifiedGallery({
        captions: false,
        rowHeight: 150,
        maxRowHeight: 150,
        margins: 10,
        border: 10,
        waitThumbnailsLoad: true,
        refreshSensitivity: 30,
        imagesAnimationDuration: 1
      })
      .on('jg.resize jg.complete', function () {
        PixelRockstarApp.EventBus.trigger("galleryResized", parseFloat($container.css("height")));
      });
  } catch (e) {
  }
}

/***************************
 * Media upload lightbox ***
 ***************************/
var PixelRockstarApp = PixelRockstarApp || {};

function pixelrockstar_uploadMediaHook() {

  if (typeof window.pixelrockstar_l10n === "undefined") {
    return;
  }

  var l10n = window._wpMediaViewsL10n || {};
  var pixel_l10n = window.pixelrockstar_l10n || {};

  /**
   * Application logic
   */

  PixelRockstarApp.Templates = {
    mainWindow: '' +
      '  <form class="pixelrockstar-Formbar pixelrockstar-FormControlBox pixelrockstar-FormControlBox--vertical pixelrockstar-searchForm js-pixelrockstar-searchForm">' +
      '    <div class="pixelrockstar-FormControl u-size2of3">' +
      '      <input type="text" value="<%= searchTerm %>" autofocus' +
      '             placeholder="' + pixel_l10n["input_placeholder_search"] + '"' +
      '             class="pixelrockstar-FormControl-input pixelrockstar-FormControl-input--big pixelrockstar-u-marginR10 js-pixelrockstar-searchTerm">' +
      '      <a class="pixelrockstar-Button pixelrockstar-Button--big pixelrockstar-Button--blue js-pixelrockstar-searchButton">' +
      pixel_l10n["button_search"] +
      '      </a>' +
      '    </div>' +
      '  </form>' +
      '  <script>jQuery(".js-pixelrockstar-searchTerm").focus();</script>',
    searchResultItem: '<img src="<%= thumbnail_url %>" alt="<%= title %>" />',
    itemPreviewPane: '' +
      '<div class="pixelrockstar-u-marginH30 pixelrockstar-u-marginV30">' +
      '  <em class="pixelrockstar-Icon pixelrockstar-Icon-close js-pixelrockstar-closeDetails"></em>' +
      '  <div class="pixelrockstar-Grid pixelrockstar-Grid-2col pixelrockstar-Grid--multiCol:60em pixelrockstar-Grid--containerAlignMiddle pixelrockstar-Grid--alignMiddle pixelrockstar-u-size1of1">' +
      '    <div class="pixelrockstar-Grid-cell">' +
      '      <div class="pixelrockstar-u-marginR30 pixelrockstar-u-textCenter">' +
      '        <img src="<%= (item.preview_url !== null) ? item.preview_url : item.thumbnail_url %>" alt="<%= item.title %>" class="pixelrockstar-searchResults-preview" />' +
      '      </div>' +
      '    </div>' +
      '    <div class="pixelrockstar-Grid-cell">' +
      '      <h3><%= item.title %></h3>' +
      '      <% if (!subscriptionError) { %>' +
      '        <% if (sizes.length > 0) { %>' +
      '          <div class="pixelrockstar-FormControlBox pixelrockstar-FormControlBox--horizontal">' +
      '            <select class="pixelrockstar-FormControl-select pixelrockstar-FormControl-select--big pixelrockstar-FormControl-select--round pixelrockstar-u-marginR5' +
      '                    js-pixelrockstar-selectedSize" style="margin: 0;">' +
      '              <% _.each(sizes, function(size) { %>' +
      '                <option value="<%= size.size %>" <% if (size.size == selectedSize) { %>selected="selected"<% } %>>' +
      '                  <%= size.title %>' +
      '                  <% if (typeof size.size_formatted !== "undefined") { %>' +
      '                    - <%= size.size_formatted %> px' +
      '                  <% } %>' +
      '                </option>' +
      '              <% }); %>' +
      '            </select>' +
      '          <% if (selectedSize == "' + PIXELROCKSTAR_CUSTOM_SIZE_VALUE + '") { %>' +
      '              <input type="text" size="4" maxlength="4" value="<%= customSize %>" id="pixelrockstar-customSize" style="flex-grow: 0;"' +
      '                     class="pixelrockstar-FormControl-input pixelrockstar-FormControl-input--big pixelrockstar-FormControl-select--round pixelrockstar-u-marginR5 js-pixelrockstar-customSize" />' +
      '           <% } %>' +
      '            <a class="pixelrockstar-Button pixelrockstar-Button--big pixelrockstar-Button--blue js-pixelrockstar-downloadButton">' +
                     pixel_l10n["button_download"] +
      '            </a>' +
      '          </div>' +
      '          <% if (canChangeDefault) { %>' +
      '            <p>' + pixel_l10n["text_sizesInfo"] + '</p>' +
      '            <div class="pixelrockstar-Checkbox pixelrockstar-u-marginT10">' +
      '              <input type="checkbox" class="js-pixelrockstar-setDefaultSize" id="pixelrockstar-setDefaultSize"' +
      '                     <%= PixelRockstarApp.Variables.checkboxSetDefaultSizeChecked ? "checked" : "" %> >' +
      '              <label class="pixelrockstar-FormControl-label" for="pixelrockstar-setDefaultSize">' +
                       pixel_l10n["label_setDefaultSize"] +
      '              </label>' +
      '              <span class="pixelrockstar-u-marginH5">&bull;</span>' +
      '              <a href="' + pixel_l10n["link_editSizePresets"] + '">' + pixel_l10n["text_editSizePresets"] + '</a>' +
      '            </div>' +
      '          <% } %>' +
      '        <% } else { %>' +
      '          <div class="pixelrockstar-u-marginB10">' +
      '            <a class="pixelrockstar-Button pixelrockstar-Button--big pixelrockstar-Button--blue js-pixelrockstar-downloadButton">' +
                     pixel_l10n["button_download"] +
      '            </a>' +
      '          </div>' +
      '        <% } %>' +
      '        <div class="pixelrockstar-Checkbox">' +
      '          <input type="checkbox" class="js-pixelrockstar-addAsFeatured" id="pixelrockstar-addAsFeatured"' +
      '                     <%= PixelRockstarApp.Variables.checkboxAddAsFeaturedChecked ? "checked" : "" %> >' +
      '          <label class="pixelrockstar-FormControl-label" for="pixelrockstar-addAsFeatured">' +
                   pixel_l10n["label_addAsFeatured"] +
      '          </label>' +
      '        </div>' +
      '      <% } else { %>' +
      '        <div class="pixelrockstar-u-fontSize14 pixelrockstar-color-red">' +
      '          <%= subscriptionError %>' +
      '        </div>' +
      '      <% } %>' +
      '      <div style="border-top: 1px solid #72777c;" class="pixelrockstar-u-marginT10 pixelrockstar-u-leading10">' + pixel_l10n["text_imageId"] + '</div>' +
      '    </div>' +
      '  </div>' +
      '</div>',
    progressBar: '' +
      '<div class="pixelrockstar-Dialog-Container pixelrockstar-Overlay pixelrockstar-Overlay--Dark pixelrockstar-Overlay--Fullscreen">' +
      '  <div class="pixelrockstar-Dialog-Content pixelrockstar-MinWidth--500">' +
      '    <div class="pixelrockstar-Dialog-Content--top"><h2><%= title %></h2></div>' +
      '    <div class="pixelrockstar-Dialog-Content--bottom">' +
      '      <div class="pixelrockstar-u-fontSize16 pixelrockstar-u-marginB20 js-pixelrockstar-progressBar--description"><%= description %></div>' +
      '      <div class="pixelrockstar-progressBar--container">' +
      '        <div class="pixelrockstar-progressBar--bar" style="width: <%= percentage %>%;"></div>' +
      '      </div>' +
      '    </div>' +
      '  </div>' +
      '</div>'
  };

  PixelRockstarApp.EventBus = _({}).extend(Backbone.Events);

  PixelRockstarApp.Variables = {
    pixelrockstar_data: {
      searchTerm: "",
      searchResults: [],
      currentPage: 1,
      totalPages: 1,
      imagesFrom: 0,
      imagesTo: 0
    },
    imagesToBeLoaded: null,
    imagesOffsetMapping: {},
    itemPreviewPane: {
      additionalOffset: 10,
      containerHeight: 0,
      itemId: null
    },
    progressBarView: null,
    mediaFrame: null,
    sizes: pixelrockstar_vars.sizes,
    defaultSize: pixelrockstar_vars.default_size,
    customSize: pixelrockstar_vars.custom_size,
    canChangeDefault: pixelrockstar_vars.can_change_default,
    imageAlignment: pixelrockstar_vars.image_alignment,
    subscriptionError: pixelrockstar_vars.subscription_error,
    checkboxSetDefaultSizeChecked: false,
    checkboxAddAsFeaturedChecked: false
  };

  /**
   * Item Model
   */
  PixelRockstarApp.Item = Backbone.Model.extend({
    defaults: {
      id: null,
      id_ext: null,
      title: "",
      thumbnail_url: null,
      thumbnail_width: null,
      thumbnail_height: null,
      preview_url: null,
      preview_width: null,
      preview_height: null
    },

    initialize: function (data) {
      data = data || {};
      if (typeof data.thumbnail === "object") {
        this.set({
          thumbnail_url: data.thumbnail.url,
          thumbnail_width: data.thumbnail.width,
          thumbnail_height: data.thumbnail.height
        });

        this.unset("thumbnail", {silent: true});
      }
      if (typeof data.preview === "object") {
        this.set({
          preview_url: data.preview.url,
          preview_width: data.preview.width,
          preview_height: data.preview.height
        });

        this.unset("preview", {silent: true});
      }
    },

    getPreviewUrl: function () {
      var self = this;

      jQuery.ajax({
        type: "get",
        dataType: "json",
        url: ajaxurl,
        data: {
          action: PIXELROCKSTAR_AJAX_ACTION,
          what: "preview",
          imageId: this.id
        },
        success: function (response) {
          self.set({preview_url: response.url});
          PixelRockstarApp.EventBus.trigger("itemPreviewUpdated")
        },
        error: function () {
          self.set({preview_url: null});
        }
      });
    },

    purchase: function (size) {
      var self = this;
      var progressBarTarget = 59;

      if (PixelRockstarApp.Variables.progressBarView) {
        PixelRockstarApp.Variables.progressBarView.close();
        PixelRockstarApp.Variables.progressBarView = null;
      }

      var model = new PixelRockstarApp.ProgressBar({
        title: pixel_l10n["progressbar_title"],
        description: pixel_l10n["progressbar_desc_prepare"],
        descriptionMapping: {
          45: pixel_l10n["progressbar_desc_optimize"],
          60: pixel_l10n["progressbar_desc_upload"],
          95: pixel_l10n["progressbar_desc_add"]
        }
      });
      PixelRockstarApp.Variables.progressBarView = new PixelRockstarApp.ProgressBarView({model: model});
      PixelRockstarApp.Variables.progressBarView.render();
      jQuery(PixelRockstarApp.Variables.progressBarView.$el).appendTo("body");
      PixelRockstarApp.Variables.progressBarView.animate(progressBarTarget, 3000);

      jQuery.ajax({
        type: "post",
        dataType: "json",
        url: ajaxurl,
        data: {
          action: PIXELROCKSTAR_AJAX_ACTION,
          what: "purchase",
          imageId: this.id,
          size: size
        },
        success: function (response) {
          if (PixelRockstarApp.Variables.progressBarView) {
            PixelRockstarApp.Variables.progressBarView.finish();
          }
          self.download({
            hash: response.hash,
            checksum: response.checksum,
            fileSize: response.fileSize
          });
        },
        error: function (response) {
          if (PixelRockstarApp.Variables.progressBarView) {
            PixelRockstarApp.Variables.progressBarView.close();
            PixelRockstarApp.Variables.progressBarView = null;
          }

          pixelrockstar_dialog({
            top:
              "<h2>" + pixel_l10n["dialog_error_title"] + "</h2>" +
              "<p>" + response.responseText + "</p>",
            bottom: '<a class="pixelrockstar-Button pixelrockstar-Button--big pixelrockstar-Button--blue js-pixelrockstar-closeDialog">OK</a>'
          });
        }
      });
    },

    download: function (purchaseData) {
      var self = this;
      var progressBarTarget = 94;

      if (PixelRockstarApp.Variables.progressBarView) {
        PixelRockstarApp.Variables.progressBarView.animate(progressBarTarget, 3000);
      }

      jQuery.ajax({
        type: "get",
        dataType: "json",
        url: ajaxurl,
        data: {
          action: PIXELROCKSTAR_AJAX_ACTION,
          what: "download",
          hash: purchaseData.hash,
          checksum: purchaseData.checksum,
          title: self.get("title")
        },
        success: function (response) {
          if (PixelRockstarApp.Variables.progressBarView) {
            PixelRockstarApp.Variables.progressBarView.finish();
          }

          self.addToLibrary(response.file);
        },
        error: function (response) {
          if (PixelRockstarApp.Variables.progressBarView) {
            PixelRockstarApp.Variables.progressBarView.close();
            PixelRockstarApp.Variables.progressBarView = null;
          }

          pixelrockstar_dialog({
            top: "<h2>" + pixel_l10n["dialog_error_title"] + "</h2>" +
              "<p>" + response.responseText + "</p>",
            bottom: '<a class="pixelrockstar-Button pixelrockstar-Button--big pixelrockstar-Button--blue js-pixelrockstar-closeDialog">OK</a>'
          });
        }
      });
    },

    addToLibrary: function (file) {
      var self = this;
      var progressBarTarget = 99;

      if (PixelRockstarApp.Variables.progressBarView) {
        PixelRockstarApp.Variables.progressBarView.animate(progressBarTarget, 1000);
      }

      jQuery.ajax({
        type: "post",
        dataType: "json",
        url: ajaxurl,
        data: {
          action: PIXELROCKSTAR_AJAX_ACTION,
          what: "add-to-library",
          file: file,
          title: self.get("title"),
          searchTerm: jQuery(".js-pixelrockstar-searchTerm").val() // Yeah, I know...
        },
        success: function (response) {
          if (jQuery('.pixelrockstar .pixelrockstar-uploadPage').length) {
            if (PixelRockstarApp.Variables.progressBarView) {
              PixelRockstarApp.Variables.progressBarView.close();
              PixelRockstarApp.Variables.progressBarView = null;
            }
            if (typeof window.ajaxurl !== "undefined") {
              window.location.href = ajaxurl.replace('admin-ajax.php', 'upload.php');
            }
          } else {
            var mediaFrame = PixelRockstarApp.Variables.mediaFrame;
            var views;

            if (self.get("setAsFeatured")) {
              self.setAsFeatured(response.attachmentId);
            }

            // Switch to Insert Media tab -> Media Library
            mediaFrame.content.mode('browse');

            // Find view with items
            views = mediaFrame.views._views['.media-frame-content'][0].views._views[''];
            for (var i = 0, viewsCount = views.length; i < viewsCount; i++) {
              var view = views[i];
              if (view.collection) {
                // Force refresh of the collection
                view.collection.props.set({
                  ignore: +(new Date())
                });
              }
            }

            if (PixelRockstarApp.Variables.progressBarView) {
              PixelRockstarApp.Variables.progressBarView.close();
              PixelRockstarApp.Variables.progressBarView = null;
            }

            // Select new item
            var selection = mediaFrame.state().get("selection");
            var attachment = wp.media.attachment(response.attachmentId);
            attachment.fetch({
              success: function () {
                //selection.reset([]);
                selection.add(attachment);

                // Set image alignment per Plugin setting
                jQuery(".attachment-display-settings").find(".alignment")
                  .val(PixelRockstarApp.Variables.imageAlignment)
                  .trigger("change");
              }
            });
          }
        },
        error: function (response) {
          if (PixelRockstarApp.Variables.progressBarView) {
            PixelRockstarApp.Variables.progressBarView.close();
            PixelRockstarApp.Variables.progressBarView = null;
          }

          pixelrockstar_dialog({
            top: "<h2>" + pixel_l10n["dialog_error_title"] + "</h2>" +
              "<p>" + response.responseText + "</p>",
            bottom: '<a class="pixelrockstar-Button pixelrockstar-Button--big pixelrockstar-Button--blue js-pixelrockstar-closeDialog">OK</a>'
          });
        }
      });
    },

    setAsFeatured: function (attachmentId) {
      wp.media.featuredImage.set(attachmentId);
    },

    setDefaultSize: function (size, customSize) {
      jQuery.ajax({
        type: "post",
        dataType: "json",
        url: ajaxurl,
        data: {
          action: PIXELROCKSTAR_AJAX_ACTION,
          what: "update-default-size",
          size: size,
          customSize: customSize
        }
      });
    }
  });

  /**
   * Item Preview Model
   */
  PixelRockstarApp.ItemPreview = Backbone.Model.extend({
    defaults: {
      item: null,
      sizes: PixelRockstarApp.Variables.sizes,
      selectedSize: PixelRockstarApp.Variables.defaultSize,
      customSize: PixelRockstarApp.Variables.customSize,
      canChangeDefault: PixelRockstarApp.Variables.canChangeDefault,
      subscriptionError: PixelRockstarApp.Variables.subscriptionError
    },

    initialize: function (data) {
      this.set("item", new PixelRockstarApp.Item(data.item || {}));
    }
  });

  PixelRockstarApp.ProgressBar = Backbone.Model.extend({
    defaults: {
      title: '',
      description: '',
      percentage: 0,
      descriptionMapping: []
    }
  });

  PixelRockstarApp.ProgressBarView = Backbone.View.extend({
    tagName: 'div',
    className: 'pixelrockstar-progressBar',
    template: _.template(PixelRockstarApp.Templates.progressBar),
    model: PixelRockstarApp.ProgressBar,

    initialize: function () {
      this.listenTo(this.model, 'change', this.updateDescription);
    },

    render: function () {
      this.$el.empty();
      this.$el.html(this.template(this.model.attributes));

      return this;
    },

    animate: function (targetWidth, speed) {
      var self = this;

      jQuery(".pixelrockstar-progressBar--bar")
        .stop(true)
        .animate(
          {
            width: targetWidth + "%"
          },
          {
            duration: speed,
            easing: "linear",
            step: function (now) {
              if (self.model.get("percentage") !== parseInt(now)) {
                self.model.set("percentage", parseInt(now));
              }
            }
          }
        );
    },

    stop: function () {
      jQuery(".pixelrockstar-progressBar--bar").stop(true);
    },

    finish: function () {
      jQuery(".pixelrockstar-progressBar--bar").stop(true, true);
    },

    close: function () {
      this.remove();
      this.unbind()
    },

    updateDescription: function () {
      var model = this.model;
      var percentage = model.get("percentage");
      var descriptionMapping = model.get("descriptionMapping");

      if (typeof descriptionMapping[percentage] === "undefined") {
        return;
      }

      this.model.set("description", descriptionMapping[percentage]);

      // Yeah, I know it's better to do `this.render()` instead of this, but render() makes the animation sad.
      jQuery(this.$el).find(".js-pixelrockstar-progressBar--description").html(this.model.get("description"));
    }
  });

  /**
   * Search Results Collection
   */
  PixelRockstarApp.SearchResultsCollection = Backbone.Collection.extend({
    model: PixelRockstarApp.Item
  });

  /**
   * Search Results View
   */
  PixelRockstarApp.SearchResultsView = Backbone.View.extend({
    tagName: 'div',
    className: 'pixelrockstar-searchResults',
    model: PixelRockstarApp.Item,

    renderItem: function (model) {
      var itemView = new PixelRockstarApp.ItemView({model: model});
      itemView.render();
      this.container.appendChild(itemView.el);

      return this;
    },

    render: function () {
      this.container = document.createDocumentFragment();

      this.collection.each(this.renderItem, this);

      this.$el.empty();
      this.$el.html(this.container);

      if (this.$('img').length > 0) {
        PixelRockstarApp.Variables.imagesToBeLoaded = this.$('img').length;
        this.$('img').one('load', _.bind(this.onLoad, this));
        setTimeout(this.showResults, 2500);
      } else {
        this.onLoad();
      }
      return this;
    },

    onLoad: function () {
      PixelRockstarApp.Variables.imagesToBeLoaded--;
      if (PixelRockstarApp.Variables.imagesToBeLoaded <= 0) {
        PixelRockstarApp.Variables.imagesToBeLoaded = null;
        this.showResults();
      }
    },

    showResults: function () {
      pixelrockstar_hideLoader();
      pixelrockstar_renderGallery();
    }
  });

  /**
   * Item View
   */
  PixelRockstarApp.ItemView = Backbone.View.extend({
    tagName: 'a',
    className: 'pixelrockstar-searchResult js-pixelrockstar-searchResult',
    template: _.template(PixelRockstarApp.Templates.searchResultItem),

    events: {
      'click': 'pixelrockstar_activeItemTrigger'
    },

    attributes: function () {
      return {
        'href': '#',
        'data-id': this.model.attributes.id
      }
    },

    render: function () {
      this.$el.html(this.template(this.model.attributes));

      return this;
    },

    pixelrockstar_activeItemTrigger: function (event) {
      event.preventDefault();

      PixelRockstarApp.EventBus.trigger('activeItem', this.model);
    }
  });

  /**
   * Item Preview Pane View
   */
  PixelRockstarApp.ItemPreviewView = Backbone.View.extend({
    tagName: 'div',
    className: 'pixelrockstar-itemPreviewPane pixelrockstar-u-isHidden',
    template: _.template(PixelRockstarApp.Templates.itemPreviewPane),
    model: PixelRockstarApp.ItemPreview,

    events: {
      'click .js-pixelrockstar-downloadButton': 'downloadImage',
      'change .js-pixelrockstar-selectedSize': 'changeSize',
      'change .js-pixelrockstar-customSize': 'changeCustomSize',
      'change .js-pixelrockstar-setDefaultSize': function () {
        PixelRockstarApp.Variables.checkboxSetDefaultSizeChecked = jQuery(".js-pixelrockstar-setDefaultSize").is(":checked")
      },
      'change .js-pixelrockstar-addAsFeatured': function () {
        PixelRockstarApp.Variables.checkboxAddAsFeaturedChecked = jQuery(".js-pixelrockstar-addAsFeatured").is(":checked")
      }
    },

    initialize: function () {
      this.listenTo(this.model, 'change', this.render);
      this.listenTo(PixelRockstarApp.EventBus, 'itemPreviewUpdated', this.updatePreview);
      this.listenTo(PixelRockstarApp.EventBus, 'imagesOffsetMappingUpdated', this.refreshPane);
    },

    render: function () {
      // Model contains item model, so we need to encode it this way
      var model = JSON.parse(JSON.stringify(this.model));

      // No item model? Nothing to display then.
      if (model.item.id === null) {
        this.$el.empty();
        return this;
      }

      this.$el.html(this.template(model));
      this.$el.removeClass("pixelrockstar-u-isHidden");

      var paneHeight = parseInt(this.$el.height()) +
        parseInt(this.$el.css("margin-top")) +
        parseInt(this.$el.css("margin-bottom")) +
        parseInt(this.$el.css("padding-top")) +
        parseInt(this.$el.css("padding-bottom")) +
        PixelRockstarApp.Variables.itemPreviewPane.additionalOffset;

      // Hide pane if changing to another image, not if re-rendering with new preview image
      if (PixelRockstarApp.Variables.itemPreviewPane.itemId !== this.model.get("item").get("id")) {
        this.resetItemsLayout();
      }

      PixelRockstarApp.Variables.itemPreviewPane.itemId = this.model.get("item").get("id");

      this.insertDetailPane(paneHeight, this.model.get("item").id);
      return this;
    },

    changeSize: function () {
      this.model.set("selectedSize", jQuery(".js-pixelrockstar-selectedSize").val());

      // Remember selected size in this relation
      PixelRockstarApp.Variables.defaultSize = this.model.get("selectedSize");
    },

    changeCustomSize: function () {
      this.model.set("customSize", jQuery(".js-pixelrockstar-customSize").val());

      // Remember selected size in this relation
      PixelRockstarApp.Variables.customSize = this.model.get("customSize");
    },

    downloadImage: function (event) {
      event.preventDefault();

      var $ = jQuery;
      var item = this.model.get("item");
      var size = this.model.get("selectedSize");
      var customSize = this.model.get("customSize");
      var updateSize = $(".js-pixelrockstar-setDefaultSize").is(":checked");

      // Begin download
      item.purchase((size !== PIXELROCKSTAR_CUSTOM_SIZE_VALUE) ? size : customSize);
      item.set("setAsFeatured", $(".js-pixelrockstar-addAsFeatured").is(":checked"));

      // Update default size on server if requested
      if (updateSize) {
        item.setDefaultSize(size, customSize);
      }
    },

    resetItemsLayout: function () {
      var $ = jQuery;
      var offsetMapping = this.getImagesOffsetMapping();

      $(".pixelrockstar-searchResult").each(function (index, element) {
        var $element = $(element);
        $element
          .removeClass("pixelrockstar-searchResults-belowDetailPane")
          .removeClass("pixelrockstar-searchResults-activeItem")
          .css("top", offsetMapping[index]);
      });
    },

    hideDetailPane: function (resetHeight) {
      this.$el.addClass("pixelrockstar-u-isHidden");
      this.resetItemsLayout();

      if (resetHeight) {
        jQuery(".justified-gallery").css("height", PixelRockstarApp.Variables.itemPreviewPane.containerHeight + "px");
      }
    },

    refreshPane: function () {
      this.hideDetailPane();
      this.render();
    },

    insertDetailPane: function (paneHeight, itemId) {
      var $ = jQuery;
      var offsetMapping = this.getImagesOffsetMapping();
      var afterPane = false;
      var itemOffset = 0;
      var paneTop = -1;

      // Create space for the pane
      $(".pixelrockstar-searchResult").each(function (index, element) {
        var $element = $(element);
        var currentItemId = $element.data("id");
        var currentOffset = element.offsetTop;

        // Find current item's position
        if (itemId == currentItemId) {
          afterPane = true;
          itemOffset = currentOffset;
          $element.addClass("pixelrockstar-searchResults-activeItem");
          paneTop = parseInt($element.css("top")) +
            parseInt($element.css("height")) +
            parseInt($element.css("margin-top")) +
            parseInt($element.css("margin-bottom")) +
            parseInt($element.css("padding-top")) +
            parseInt($element.css("padding-bottom")) +
            PixelRockstarApp.Variables.itemPreviewPane.additionalOffset;
        }

        // Move all images on subsequent lines
        if (afterPane && currentOffset > itemOffset) {
          $element
            .addClass("pixelrockstar-searchResults-belowDetailPane")
            .css("top", offsetMapping[index] + paneHeight + "px");
        }
      });

      // Move pane on position
      this.$el.css("top", paneTop + "px");

      // Update gallery height
      $(".justified-gallery").css("height", (PixelRockstarApp.Variables.itemPreviewPane.containerHeight + paneHeight) + "px");
    },

    updatePreview: function () {
      var self = this;
      var $image = jQuery("<img />");

      $image
        .attr("src", this.model.get("item").get("preview_url"))
        .load(function () {
          self.render();
          this.remove();
        });
    },

    getImagesOffsetMapping: function (refresh) {
      var offsetMapping = PixelRockstarApp.Variables.imagesOffsetMapping;

      if (Object.keys(offsetMapping).length !== 0 && !refresh) {
        return offsetMapping;
      }

      offsetMapping = [];
      jQuery(".pixelrockstar-searchResult").each(function (index, element) {
        offsetMapping[index] = element.offsetTop;
      });

      PixelRockstarApp.Variables.imagesOffsetMapping = offsetMapping;
      PixelRockstarApp.EventBus.trigger('imagesOffsetMappingUpdated');

      return offsetMapping
    }
  });

  /**************************************************/

  /**
   * Contains main panel view
   */
  wp.media.view.PixelRockstar = wp.media.View.extend({
    className: 'media-pixelrockstar',

    // bind view events
    events: {
      'submit .js-pixelrockstar-searchForm': 'pixelrockstar_search',
      'click .js-pixelrockstar-searchButton': 'pixelrockstar_search',
      'click .js-pixelrockstar-closeDetails': 'pixelrockstar_closeItemPreviewPane',
      'click .js-pixelrockstar-searchBack': 'pixelrockstar_previousPage',
      'click .js-pixelrockstar-searchNext': 'pixelrockstar_nextPage'
    },

    initialize: function () {
      this.props = new Backbone.Model({
        pixelrockstar_data: _.clone(PixelRockstarApp.Variables.pixelrockstar_data)
      });

      this.listenTo(this.props, 'change:pixelrockstar_data', this.render);
      this.listenTo(PixelRockstarApp.EventBus, 'activeItem', this.pixelrockstar_renderItemPreviewPane);
      this.listenTo(PixelRockstarApp.EventBus, 'galleryResized', this.pixelrockstar_galleryResized);

      this.searchResultsCollection = new PixelRockstarApp.SearchResultsCollection();
      this.searchResultsView = new PixelRockstarApp.SearchResultsView({
        collection: this.searchResultsCollection
      });
      this.itemPreviewView = new PixelRockstarApp.ItemPreviewView({
        model: new PixelRockstarApp.ItemPreview({})
      });
    },

    render: function () {
      var template = _.template(PixelRockstarApp.Templates.mainWindow);
      var data = this.props.get('pixelrockstar_data');

      this.$el.empty();
      this.$el.html(template({searchTerm: data.searchTerm}));

      // Reset
      var $container = jQuery("<div />").addClass("pixelrockstar-searchContainer");
      this.searchResultsCollection.reset();
      this.itemPreviewView.remove();
      this.itemPreviewView = new PixelRockstarApp.ItemPreviewView({
        model: new PixelRockstarApp.ItemPreview({})
      });

      for (var index in data.searchResults) {
        var item = new PixelRockstarApp.Item(data.searchResults[index]);
        this.searchResultsCollection.add(item);
      }

      $container.append(this.searchResultsView.render().el);
      $container.append(this.itemPreviewView.render().el);

      this.$el.append($container);

      var imagesString = '';
      var classBackButton = "";
      var classNextButton = "";

      if (data.totalPages > 1) {
        if (data.imagesFrom > 0) {
          imagesString = ' &bull; Images ' + data.imagesFrom + ' to ' + data.imagesTo;
        }
        if (data.currentPage <= 1) {
          classBackButton = "pixelrockstar-u-isDisabled";
        }
        if (data.currentPage >= data.totalPages) {
          classNextButton = "pixelrockstar-u-isDisabled";
        }
        html =
          '<div class="pixelrockstar-Toolbar pixelrockstar-Grid pixelrockstar-Grid--multiCol:60em pixelrockstar-Grid--3col:60em pixelrockstar-Grid--alignMiddle">' +
          '  <div class="pixelrockstar-Grid-cell pixelrockstar-u-spacingR30">' +
          '    <div class="pixelrockstar-u-fontSize14">' +
          '      Page ' + data.currentPage + ' of ' + data.totalPages +
          imagesString +
          '    </div>' +
          '  </div>' +
          '  <div class="pixelrockstar-Grid-cell pixelrockstar-u-spacingR30">' +
          '    <div class="pixelrockstar-u-textCenter">' +
          '      <a class="pixelrockstar-Button pixelrockstar-Button--big pixelrockstar-Button--white ' + classBackButton + ' pixelrockstar-u-marginR20 js-pixelrockstar-searchBack">' +
          pixel_l10n["button_back"] +
          '      </a>' +
          '      <a class="pixelrockstar-Button pixelrockstar-Button--big pixelrockstar-Button--blue ' + classNextButton + ' js-pixelrockstar-searchNext">' +
          pixel_l10n["button_next"] +
          '      </a>' +
          '    </div>' +
          '  </div>' +
          '</div>';
        this.$el.append(html);
      }
      return this;
    },

    pixelrockstar_search: function (event) {
      event.preventDefault();
      var searchTerm = this.$(".js-pixelrockstar-searchTerm").val();
      this.pixelrockstar_search2(searchTerm);
    },

    pixelrockstar_search2: function (searchTerm, page) {
      var self = this;
      var data = _.clone(PixelRockstarApp.Variables.pixelrockstar_data);
      if (!jQuery('.pixelrockstar .pixelrockstar-uploadPage').length) {
        var $container = jQuery(".media-modal").first();
      }

      page = page || 1;
      data.searchTerm = searchTerm;

      pixelrockstar_showLoader("large", "black", $container);

      jQuery.ajax({
        type: "get",
        dataType: "json",
        url: ajaxurl,
        data: {
          "action": PIXELROCKSTAR_AJAX_ACTION,
          "what": "search",
          "searchTerm": searchTerm,
          "page": page
        },
        success: function (response) {
          data.searchResults = response.searchResults;
          data.currentPage = parseInt(response.currentPage);
          data.totalPages = parseInt(response.totalPages);
          data.imagesFrom = parseInt(response.imagesFrom);
          data.imagesTo = parseInt(response.imagesTo);
          data.random = Math.random(); // prevent the loader to get stuck on repeated query

          self.props.set('pixelrockstar_data', data);
        },
        error: function (response) {
          pixelrockstar_hideLoader();
          pixelrockstar_dialog(
            {
              top: "<h2>" + pixel_l10n["dialog_error_title"] + "</h2>" +
                "<p>" + response.responseText + "</p>",
              bottom: '<a class="pixelrockstar-Button pixelrockstar-Button--big pixelrockstar-Button--blue js-pixelrockstar-closeDialog">OK</a>'
            }
          );
        },
        complete: function () {
          // Reset images offset mapping
          PixelRockstarApp.Variables.imagesOffsetMapping = {};
        }
      });
    },

    pixelrockstar_changePageAction: function (offset) {
      var data = _.clone(this.props.get('pixelrockstar_data'));
      var page = data.currentPage + offset;

      if (data.searchTerm === "" || page < 1) {
        return;
      }

      this.pixelrockstar_search2(data.searchTerm, page);
    },
    pixelrockstar_renderItemPreviewPane: function (item) {
      if (item.get("preview_url") === null) {
        item.getPreviewUrl();
      }

      if (this.itemPreviewView.model.get("item") == item) {
        this.pixelrockstar_closeItemPreviewPane();
        return;
      }

      this.itemPreviewView.model.set("item", item);
    },

    pixelrockstar_closeItemPreviewPane: function () {
      this.itemPreviewView.model.set("item", new PixelRockstarApp.Item({}));
      this.itemPreviewView.hideDetailPane(true);
    },

    pixelrockstar_galleryResized: function (height) {
      if (PixelRockstarApp.Variables.itemPreviewPane.containerHeight == height) {
        return;
      }

      PixelRockstarApp.Variables.itemPreviewPane.containerHeight = height;

      this.itemPreviewView.getImagesOffsetMapping(true);
    },

    pixelrockstar_previousPage: function () {
      if (this.$(".js-pixelrockstar-searchBack").hasClass("pixelrockstar-u-isDisabled")) {
        return;
      }
      this.pixelrockstar_changePageAction(-1);
    },

    pixelrockstar_nextPage: function () {
      if (this.$(".js-pixelrockstar-searchNext").hasClass("pixelrockstar-u-isDisabled")) {
        return;
      }
      this.pixelrockstar_changePageAction(+1);
    }
  });

  // Replace the default MediaFrame.Post view
  var Post = wp.media.view.MediaFrame.Post;
  var ImageDetails = wp.media.view.MediaFrame.ImageDetails;

  wp.media.view.MediaFrame.Post = Post.extend({
    initialize: function () {
      PixelRockstarApp.Variables.mediaFrame = this;
      Post.prototype.initialize.apply(this, arguments);
      this.on('open', this.pixelrockstarDirectOpen, this);
      this.on('content:create:pixelrockstarRouter', this.pixelrockstarRouter, this);
    },
    browseRouter: function (routerView) {
      //Post.prototype.browseRouter.apply(this, arguments);
      routerView.set({
        upload: {
          text: l10n.uploadFilesTitle,
          priority: 20
        },
        browse: {
          text: l10n.mediaLibraryTitle,
          priority: 40
        },
        pixelrockstarRouter: {
          text: pixel_l10n['title2'],
          priority: 60
        }
      });
    },
    pixelrockstarRouter: function () {
      this.$el.removeClass('hide-toolbar');
      this.content.set(new wp.media.view.PixelRockstar({
        controller: this
      }));
    },
    pixelrockstarDirectOpen: function () {
      if (jQuery(PIXELROCKSTAR_BUTTON_ADD_STOCK_PHOTO).attr("data-button-clicked") != "1") {
        return;
      }
      jQuery(PIXELROCKSTAR_BUTTON_ADD_STOCK_PHOTO).removeAttr("data-button-clicked");
      this.content.mode('pixelrockstarRouter');
    }
  });

  wp.media.view.MediaFrame.ImageDetails = ImageDetails.extend({
    initialize: function () {
      PixelRockstarApp.Variables.mediaFrame = this;
      ImageDetails.prototype.initialize.apply(this, arguments);
      this.on('content:create:pixelrockstarRouter', this.pixelrockstarRouter, this);
    },
    browseRouter: function (routerView) {
      //ImageDetails.prototype.browseRouter.apply(this, arguments);
      routerView.set({
        upload: {
          text: l10n.uploadFilesTitle,
          priority: 20
        },
        browse: {
          text: l10n.mediaLibraryTitle,
          priority: 40
        },
        pixelrockstarRouter: {
          text: pixel_l10n['title2'],
          priority: 60
        }
      });
    },
    pixelrockstarRouter: function () {
      this.$el.removeClass('hide-toolbar');
      this.content.set(new wp.media.view.PixelRockstar({
        controller: this
      }));
    }
  });

  if (jQuery('.pixelrockstar .pixelrockstar-uploadPage').length) {
    var UploadPage = new wp.media.view.PixelRockstar();
    UploadPage.$el = jQuery('.pixelrockstar .pixelrockstar-uploadPage');
    $(document).on('submit', '.pixelrockstar .pixelrockstar-uploadPage .js-pixelrockstar-searchForm', function (event) {
      UploadPage.pixelrockstar_search(event);
    });
    $(document).on('click', '.pixelrockstar .pixelrockstar-uploadPage .js-pixelrockstar-searchButton', function (event) {
      UploadPage.pixelrockstar_search(event);
    });
    $(document).on('click', '.pixelrockstar .pixelrockstar-uploadPage .js-pixelrockstar-searchBack', function (event) {
      UploadPage.pixelrockstar_previousPage(event);
    });
    $(document).on('click', '.pixelrockstar .pixelrockstar-uploadPage .js-pixelrockstar-searchNext', function (event) {
      UploadPage.pixelrockstar_nextPage(event);
    });
    $(document).on('click', '.pixelrockstar .pixelrockstar-uploadPage .js-pixelrockstar-closeDetails', function (event) {
      UploadPage.pixelrockstar_closeItemPreviewPane(event);
    });
  }

}
