<?php
/**
 * @package PixelRockstar
 */
/*
Plugin Name: PixelRockstar
Plugin URI: https://www.pixelrockstar.com/
Description: Get legally safe﻿, high-quality images into WordPress. Search, buy & add stock photos in seconds. One click needed to add images to your articles or blog posts. 4,000,000 legally safe images to use forever in your website and social media.
Author: PixelRockstar
Version: 1.1.2
Tested up to: 5.0.3
Text Domain: pixelrockstar
Domain Path: /languages
*/

namespace PixelRockstar;

// Make sure we don't expose any info if called directly
if (!function_exists('add_action')) {
    die("Nah, that won't work.");
}

define('PIXELROCKSTAR_PLUGIN_NAME', 'PixelRockstar');
define('PIXELROCKSTAR_VERSION', '1.1.2');
define('PIXELROCKSTAR_MINIMUM_WP_VERSION', '4.0');
define('PIXELROCKSTAR_PLUGIN_URL', plugins_url('', __FILE__));
define('PIXELROCKSTAR_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('PIXELROCKSTAR_PLUGIN_BASENAME', plugin_basename(__FILE__));
define('PIXELROCKSTAR_URL', 'https://www.pixelrockstar.com/');
define('PIXELROCKSTAR_CONTACT_EMAIL', 'hello@pixelrockstar.com');

define('PIXELROCKSTAR_RATE_US_URL', 'https://wordpress.org/support/plugin/pixelrockstar/reviews/#new-post');
define('PIXELROCKSTAR_LICENSE_URL', 'https://www.pixelrockstar.com/license-agreement/');
define('PIXELROCKSTAR_TERMS_OF_SERVICE_URL', 'https://www.pixelrockstar.com/terms-of-service/');
define('PIXELROCKSTAR_PRIVACY_POLICY_URL', 'https://www.pixelrockstar.com/privacy-policy/');
define('PIXELROCKSTAR_FACEBOOK_URL', 'https://www.facebook.com/PixelRockstar/');
define('PIXELROCKSTAR_TWITTER_URL', 'https://twitter.com/PixelRockstarhq');

define('PIXELROCKSTAR_IMAGE_META_KEY', '_pixelrockstar_image_data');

add_action('wp_loaded', array('\PixelRockstar\PixelRockstar', 'init'));

// Register class autoloader
spl_autoload_register('\PixelRockstar\autoloader');
function autoloader($className)
{
    if (strpos($className, 'PixelRockstar\\') === 0) {
        $classesDir = PIXELROCKSTAR_PLUGIN_DIR . 'classes' . DIRECTORY_SEPARATOR;
        $classFilename = preg_replace('%^PixelRockstar\\\%', '', $className) . ".php";
        $classFilename = str_replace('\\', DIRECTORY_SEPARATOR, $classFilename);
        if (file_exists($classesDir . $classFilename)) {
            require_once($classesDir . $classFilename);
        } else {
            pluginForciblyDeactivatedMessage();

            require_once(ABSPATH . 'wp-admin/includes/plugin.php');
            \deactivate_plugins(__FILE__);
        }
    }
}

/*********************************************************************************************************************
 ** GENERIC FUNCTIONS ************************************************************************************************
 *********************************************************************************************************************/

/**
 * Renders page
 *
 * @param string $page        Page filename
 * @param array  $params      Parameters to be provided to the page
 * @param bool   $inContainer If true, page is rendered into a container `<div>`
 */
function renderPage($page, $params = array(), $inContainer = true)
{
    $path = PIXELROCKSTAR_PLUGIN_DIR . "/pages/";

    if ($inContainer) {
        echo '<div class="pixelrockstar wrap">';
    }

    if (file_exists($path . $page)) {
        include($path . $page);
    } else {
        echo "Page not found.";
    }

    if ($inContainer) {
        echo '</div>';
    }
}

/**
 * Generates ID for HTML elements
 *
 * @param string $id
 *
 * @return string
 */
function generateId($id)
{
    return PixelRockstar::$appId . "_" . $id;
}

function getPostValue($name, $default = "")
{
    return (isset($_POST[$name])) ? sanitize_text_field($_POST[$name]) : $default;
}

function getGetValue($name, $default = "")
{
    return (isset($_GET[$name])) ? sanitize_text_field($_GET[$name]) : $default;
}

function pluginForciblyDeactivatedMessage()
{
    ?>
    <div class="error notice">
        <p>
            <?php _e(
                'The PixelRockstar plugin is misconfigured and was disabled. Sorry for inconvenience.',
                PixelRockstar::$languageDomain
            ); ?>
        </p>
    </div>
    <?php
}

/**
 * Renders a popup dialog
 *
 * @param array $texts    Array with texts; keys must match any of CSS classes `pixelrockstar-Dialog-Content--*`.
 *                        Content will be printed from the top to the bottom
 * @param bool  $hidden   Default state of the dialog
 * @param array $settings Additional settings, accepted keys:
 *                        (string) class - Container's CSS class
 */
function dialog(array $texts, $hidden = true, $settings = array())
{
    $containerClass = ($hidden) ? "pixelrockstar-u-isHidden" : "";
    if (isset($settings["class"])) {
        $containerClass .= " " . $settings["class"];
    }
    ?>
    <div class="pixelrockstar-Dialog-Container pixelrockstar-Overlay pixelrockstar-Overlay--Dark pixelrockstar-Overlay--Fullscreen <?php echo $containerClass; ?>">
        <div class="pixelrockstar-Dialog-Content">
            <?php
            foreach ($texts as $class => $text) {
                ?>
                <div class="pixelrockstar-Dialog-Content--<?php echo $class; ?>">
                    <?php echo $text; ?>
                </div>
                <?php
            }
            ?>
        </div>
    </div>
    <?php
}
